# generic terrain following radar by Warty (Vulcan version - 2024)
# 250 knots = 422 fps = 25,320 fpm
# so an object that is 10 seconds away is 4,220 ft in the distance.
# This might not be far enough to clear, even using AP climb rate (Vertical speed)!
var tfr_loop = func {
    if(getprop("/afcs/tfr/engaged") == 1 ){
    	#var  clearance = getprop("/afcs/tfr/tgtClearance-ft") ;
    	var  clearance = getprop("/autopilot/settings/target-agl-ft") ;
		var      myLat = getprop("/position/latitude-deg") ;
		var      myLon = getprop("/position/longitude-deg");
		var      myAlt = math.round(getprop("/position/altitude-ft") ) ;	
		var northSpeed = getprop("/velocities/speed-north-fps") ;
		var  eastSpeed = getprop("/velocities/speed-east-fps") ; 
		var secondsAhead = 10 ;   
		tfr_lookahead(secondsAhead, clearance, myLat, myLon, myAlt, northSpeed, eastSpeed);
	}
	settimer(tfr_loop, 0.25);
}
var tfr_lookahead = func(lookaheads, clearance, myLat, myLon, myAlt, northSpeed, eastSpeed) {
	var highest = 0 ;  var maxFps = 0 ;
	for(var i = 0; i <= lookaheads; i += 1)
    { 
    	var    DEG2RAD = 0.016774532925 ;
		var      FT2NM = 0.000164578833554 ;
		var  Ft2degLat = FT2NM /60 ; # 0.000002742980559
		var    nextLat = myLat +   (i+1) * northSpeed * Ft2degLat ;
		var    nextLon = myLon +   (i+1) * eastSpeed  * Ft2degLat * math.cos(DEG2RAD*myLat) ;
		var       info = geodinfo(nextLat,  nextLon)  ;
		if ( info == nil ){ return ; }
		var     tgtAlt = (info[0] * 3.280839895) + clearance ;
		var     tgtFps = (tgtAlt - myAlt)/(1+i*0.8) ; # 1 sec = /0.8 , 10s = /8.2
		tgtAlt = 100 * math.round(tgtAlt * 0.01) ; # nearest 100
		tgtFps =  10 * math.round(tgtFps * 0.10) ; # nearest 10fps = 600fpm
		# prioritise those further away, for clearance in advance
		if(tgtFps > 10){ tgtFps = tgtFps + i*5 ; }
    	var fps_prop = sprintf("/afcs/tfr/tgtAlt-fps[%s]/", i );
    	setprop(fps_prop, tgtFps );
    	if(i == 0){ highest = tgtAlt ;  maxFps = tgtFps ; }
    	if( tgtAlt > highest ){ highest = tgtAlt ; }
    	if( tgtFps > maxFps  ){ maxFps  = tgtFps ; }
    }
    setprop("/autopilot/settings/target-altitude-ft", highest);
	setprop("/autopilot/settings/vertical-speed-fpm", maxFps*60);
}
setlistener("/sim/signals/fdm-initialized", func { # ====================== initialization
    setprop("/afcs/tfr/engaged", 0);
    setprop("/afcs/tfr/tgtClearance-ft", 600); # no longer needed ???
    setprop("/autopilot/settings/target-agl-ft", 500);
	tfr_loop();  
})
